<?php
/**
 * A controller for handling sent forms
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2011 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class WebDoFormController extends BaseController implements Controller {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$resolverInstance		An instance of a resolver class
	 * @return	$controllerInstance		A prepared instance of this class
	 */
	public static final function createWebDoFormController (CommandResolver $resolverInstance) {
		// Create the instance
		$controllerInstance = new WebDoFormController();

		// Set resolver instance
		$controllerInstance->setResolverInstance($resolverInstance);

		// We need the controller instance in resolver class so set it here
		$resolverInstance->setControllerInstance($controllerInstance);

		// Return the prepared instance
		return $controllerInstance;
	}

	/**
	 * Handles the given request and response
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	public function handleRequest (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the "form action"
		$formAction = $requestInstance->getRequestElement('form');

		// Get command instance from resolver
		$commandInstance = $this->getResolverInstance()->resolveCommand($formAction);

		// Add more filters by the command
		$commandInstance->addExtraFilters($this, $requestInstance);

		// Try to run the pre filters, if auth exceptions come through redirect here
		try {
			// Run the pre filters
			$this->executePreFilters($requestInstance, $responseInstance);
		} catch (UserAuthorizationException $e) {
			// Redirect to main page
			$responseInstance->redirectToConfiguredUrl('login_failed');

			// Exit here
			exit();
		}

		// Is the request still valid? Post filters shall only be executed of
		// the request is valid
		if ($requestInstance->isRequestValid()) {
			// Execute the command
			$commandInstance->execute($requestInstance, $responseInstance);

			// Execute *very* generic ppost filters
			$this->executePostFilters($requestInstance, $responseInstance);
		}

		// Flush the buffer out
		$responseInstance->flushBuffer();
	}
}

// [EOF]
?>
